/*
   cfg.c -- PIKT config file routines

   $Header: /home/cvs/ucgsb/pikt/src/piktc/cfg.c,v 1.11 1999/05/05 19:32:45 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "common.h"

int firstalarm;

/* register a macro/alarm def component
 */
void
regcomp (char *component)
{
	def *ndfp;

	if (isdebug)
		logreturn(PIKTLOG, DEBUG, __FILE__, __LINE__, FALSE, "regcomping COMP %s", component);
	ndfp = (def *) malloc(sizeof(def));
	if (!ndfp)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in regcomp(), malloc() failure for ndfp");
	ndfp->next = (def *) NULL;
	ndfp->component = component;
	if (!gidp->definition)
		gidp->definition = ndfp;
	else
		defcurr->next = ndfp;
	defcurr = ndfp;
	return;
}

/* begin writing the pikt .alt file
 */
void
beginalarms (char *alertname)
{
	static char *outfile;
	nam *naltp;
	char confline[MAXLINE+1];

	outfile = (char *) malloc((strlen(stgdir)+strlen(alertname)+6) * sizeof(char));
	sprintf(outfile, "%s/%s.alt", stgdir, alertname);
	if ((cfgout = fopen(outfile, "w")) == NULL) {
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in beginalarms(), fopen() error for %s", outfile);
	}
	free(outfile);
	naltp = (nam *) malloc(sizeof(nam));
	if (!naltp)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in beginalarms(), malloc() failure for naltp");
	naltp->next = (nam *) NULL;
	naltp->name = strdup(alertname);
	naltp->state = ADD;
	if (!alerts)
		alerts = naltp;
	else {
		naltp->next = alerts;
		alerts = naltp;
	}
	if (nicecmd)
		sprintf(confline, "%s %s %s %s %s %s %s/pikt", mins, hrs, dom, mons, dow, nicecmd, bindir);
	else
		sprintf(confline, "%s %s %s %s %s %s/pikt", mins, hrs, dom, mons, dow, bindir);
	if (mailcmd) {
		strcat(confline, " +M ");
		strcat(confline, "\"");
		strcat(confline, mailcmd);
		strcat(confline, "\"");
	}
	if (lpcmd) {
		strcat(confline, " +L ");
		strcat(confline, "\"");
		strcat(confline, lpcmd);
		strcat(confline, "\"");
	}
	strcat(confline, " +A ");
	strcat(confline, alertname);
	naltp->aux = strdup(confline);
	naltp->mode = (char *) NULL;
	naltp->uid = (char *) NULL;
	naltp->gid = (char *) NULL;
	firstalarm = TRUE;
}

/* write an alarm definition to the pikt .alt file
 */
void
writealarm (char *alarmname)
{
	ident *idp;
	def *dp;
	const char s1[] = " init begin rule end ";
	const char s2[] = " status level task input filter seps separators keys dat data do if elif elsif elseif else endif fi while endwhile repeat until for endfor switch endswitch endsw breakswitch breaksw case default break cont continue again redo next skip last pause quit die output set exec ";
	static char *comp;

	idp = lookupident(alarmname, ALMCFG);
	if (idp) {
		if (firstalarm)
			firstalarm = FALSE;
		else
			fprintf(cfgout,"\n\n");
		fprintf(cfgout,"%s",alarmname);
		foreach (dp, idp->definition) {
			comp = (char *) malloc(strlen(dp->component)+2+1);
			sprintf(comp, " %s ", dp->component);
			if (strstr(s1, comp))
				fprintf(cfgout, "\n\t%s ", dp->component);
			else if (strstr(s2, comp) || !strcmp(" for ( ",comp) || !strcmp(" switch ( ",comp))
				fprintf(cfgout, "\n\t\t%s ", dp->component);
			else if (!strcmp("[",dp->component) || (dp->next && (!strcmp("[",dp->next->component) || !strcmp("]",dp->next->component))))
				fprintf(cfgout, "%s", dp->component);
			else
				fprintf(cfgout, "%s ", dp->component);
			free(comp);
		}
	} else
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in writealarm(), %s, %s alarm not registered", cfgfilecurr, alarmname);
	return; 
}

/* close the pikt .alt file
 */
void
endalarms (char *alert)
{

	fprintf(cfgout,"\n");
	if (fclose(cfgout))
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in endalarms(), fclose() error for %s/%s.alt", altdir, alert);
	return;
}

/* begin writing the other/program file
 */
void
beginfile (char *filpath, char *mode, char *uid, char *gid, nam **list)
{
	char *cp;
	char *dir, *fil;
	static char *outfile;
	nam *nfilp;

	if ((cp = strrchr(filpath, '/'))) {
		*cp = '\0';
		dir = strdup(filpath);
		fil = strdup(cp+1);
	} else {
		dir = strdup("");
		fil = strdup(filpath);
	}
	outfile = (char *) malloc((strlen(stgdir)+strlen(fil)+2) * sizeof(char));
	sprintf(outfile, "%s/%s", stgdir, fil);
	if ((cfgout = fopen(outfile, "w")) == NULL)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in beginfile(), fopen() error for %s", outfile);
	free(outfile);
	nfilp = (nam *) malloc(sizeof(nam));
	if (!nfilp)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in beginfile(), malloc() failure for nfilp");
	nfilp->next = (nam *) NULL;
	nfilp->name = fil;
	nfilp->aux = dir;
	nfilp->mode = mode;
	nfilp->uid = uid;
	nfilp->gid = gid;
	nfilp->state = ADD;
	if (*list)
		nfilp->next = *list;
	*list = nfilp;
	return;
}

/* write a single line to the other/program file
 */
void
writefile (char *line)
{

	fprintf(cfgout, "%s\n", line);
	return;
}

/* close the other/program file
 */
void
endfile (char *fil)
{
	char *cp;

	if ((cp = strrchr(fil, '/')))
		cp = cp+1;
	else
		cp = fil;
	if (fclose(cfgout))
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in endfile(), fclose() error for %s/%s", stgdir, cp);
	return;
}

/* begin writing the objectfile
 */
void
beginobjects (char *objectname)
{
	static char *outfile;
	nam *nobjp;

	outfile = (char *) malloc((strlen(stgdir)+strlen(objectname)+6) * sizeof(char));
	sprintf(outfile, "%s/%s.obj", stgdir, objectname);
	if ((cfgout = fopen(outfile, "w")) == NULL)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in beginobjects, fopen() error for %s", outfile);
	free(outfile);
	nobjp = (nam *) malloc(sizeof(nam));
	if (!nobjp)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in beginobjects(), malloc() failure for nobjp");
	nobjp->next = (nam *) NULL;
	nobjp->name = strdup(objectname);
	nobjp->aux = (char *) NULL;
	nobjp->mode = (char *) NULL;
	nobjp->uid = (char *) NULL;
	nobjp->gid = (char *) NULL;
	nobjp->state = ADD;
	if (!objects)
		objects = nobjp;
	else {
		nobjp->next = objects;
		objects = nobjp;
	}

}

/* write a single object to the objectfile
 */
void
writeobject (char *object)
{

	fprintf(cfgout,"%s\n",object);
}

/* close the objectfile
 */
void
endobjects (char *objs)
{

	if (fclose(cfgout))
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in endobjects(), fclose error for %s/%s.obj", objdir, objs);
	return;
}

/* return the length of the piktd.conf line for the given alert
 */
int
conflinelen (char *alertname)
{
	int len;

	len = strlen(pikt)+1;			/* e.g., '/pikt/bin/pikt ' */
	len += strlen(mins) + 1 +		/* 'x ' */
	       strlen(hrs) + 1 +		/* 'x ' */
	       strlen(dom) + 1 +		/* 'x ' */
	       strlen(mons) + 1 +		/* 'x ' */
	       strlen(dow) + 1 ;		/* 'x ' */
	if (nicecmd)
		len += 1 + strlen(nicecmd);
	if (mailcmd)
		len += 6 + strlen(mailcmd);	/* '+M "..." ' */
	if (lpcmd)
		len += 6 + strlen(lpcmd);	/* '+P "..." ' */
	len += 4 + strlen(alertname);		/* '+A ...\n' */
	return len;
}
