%{

/*
   cmd.l -- PIKT piktc command-line lexer

   $Header: /home/cvs/ucgsb/pikt/src/piktc/cmd.l,v 1.12 1999/05/09 18:39:48 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "common.h"
#include "cmd.tab.h"

char errmsg[MAXLINE+1];
int cmdinput(void);

char ALL[] = "+A all +P all +F all +O all +H all ";
int ii;

#undef YY_INPUT
#define YY_INPUT(buf, result, max_size)	{ \
		char c = cmdinput(); \
		result = (c == EOF) ? YY_NULL : (buf[0] = c, 1); \
	}

#define CHKCONFLICT(A, B, FL)	if (FL) {\
					fprintf(stderr, "conflicting options: %c %c\n", A, B);\
					bailout = TRUE;\
				}

%}

W		[ \t]
D		[0-9]
L		[A-Za-z]
DL		[0-9A-Za-z]

%x MS DS AS PS FS OS HS

%option noyywrap

%%

<INITIAL,DS,AS,PS,FS,OS,HS>"-"	{ BEGIN(MS);
				  return _MINUS;
				}
<INITIAL,DS,AS,PS,FS,OS,HS>"+"	{ BEGIN(MS);
				  return _PLUS;
				}

<MS>c|C				{ if (yytext[0] == 'c') {
					if (!postscan) {
						CHKCONFLICT('c', 'C', isdoublecheck);
						ischeck = TRUE;
					}
				  } else {
					if (!postscan) {
						CHKCONFLICT('C', 'c', ischeck);
						isdoublecheck = TRUE;
					}
				  }
				  if (!postscan) {
					  CHKCONFLICT('c', 'd', isdisable);
					  CHKCONFLICT('c', 'e', isenable);
					  CHKCONFLICT('c', 'h', ishelp);
					  CHKCONFLICT('c', 'i', isinstall);
					  CHKCONFLICT('c', 'k', iskillpiktd);
					  CHKCONFLICT('c', 'K', iskillpiktc_svc);
					  CHKCONFLICT('c', 'l', islist);
					  CHKCONFLICT('c', 'r', isrestartpiktd);
					  CHKCONFLICT('c', 'R', isrestartpiktc_svc);
					  CHKCONFLICT('c', 's', isshow);
					  CHKCONFLICT('c', 't', isdelete);
				  }
				  return _OPT;
				}

<MS>d				{ if (!postscan) {
					isdisable = TRUE;
					CHKCONFLICT('d', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('d', 'e', isenable);
					CHKCONFLICT('d', 'h', ishelp);
					CHKCONFLICT('d', 'i', isinstall);
					CHKCONFLICT('d', 'K', iskillpiktc_svc);
					CHKCONFLICT('d', 'l', islist);
					CHKCONFLICT('d', 'R', isrestartpiktc_svc);
					CHKCONFLICT('d', 's', isshow);
				  }
				  return _OPT;
				}

<MS>D				{ BEGIN(DS);
				  if (!postscan)
					  CHKCONFLICT('D', 'h', ishelp);
				  return _DEFOPT;
				}

<MS>e				{ if (!postscan) {
					isenable = TRUE;
					CHKCONFLICT('e', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('e', 'd', isdisable);
					CHKCONFLICT('e', 'h', ishelp);
					CHKCONFLICT('e', 'K', iskillpiktc_svc);
					CHKCONFLICT('e', 'l', islist);
					CHKCONFLICT('e', 'R', isrestartpiktc_svc);
					CHKCONFLICT('e', 's', isshow);
					CHKCONFLICT('e', 't', isdelete);
				  }
				  return _OPT;
				}

<MS>F				{ BEGIN(FS);
				  if (!postscan) {
					CHKCONFLICT('F', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('F', 'h', ishelp);
				  }
				  return _FILOPT;
				}

<MS>Ff				{ BEGIN(FS);
				  if (!postscan) {
					CHKCONFLICT('F', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('F', 'h', ishelp);
					stripfil = FALSE;
				  }
				  return _FILOPT;
				}

<MS>O				{ BEGIN(OS);
				  if (!postscan) {
					CHKCONFLICT('O', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('O', 'h', ishelp);
				  }
				  return _OBJOPT;
				}

<MS>Of				{ BEGIN(OS);
				  if (!postscan) {
					CHKCONFLICT('O', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('O', 'h', ishelp);
					stripobj = FALSE;
				  }
				  return _OBJOPT;
				}

<MS>P				{ BEGIN(PS);
				  if (!postscan) {
					CHKCONFLICT('P', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('P', 'h', ishelp);
				  }
				  return _PRGOPT;
				}

<MS>Pf				{ BEGIN(PS);
				  if (!postscan) {
					CHKCONFLICT('P', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('P', 'h', ishelp);
					stripprg = FALSE;
				  }
				  return _PRGOPT;
				}

<MS>G				{ if (!postscan)
					isdebug = TRUE;
				  return _OPT;
				}

<MS>h				{ if (!postscan) {
					ishelp = TRUE;
					CHKCONFLICT('h', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('h', 'd', isdisable);
					CHKCONFLICT('h', 'e', isenable);
					CHKCONFLICT('h', 'i', isinstall);
					CHKCONFLICT('h', 'k', iskillpiktd);
					CHKCONFLICT('h', 'K', iskillpiktc_svc);
					CHKCONFLICT('h', 'l', islist);
					CHKCONFLICT('h', 'r', isrestartpiktd);
					CHKCONFLICT('h', 'R', isrestartpiktc_svc);
					CHKCONFLICT('h', 's', isshow);
					CHKCONFLICT('h', 't', isdelete);
					CHKCONFLICT('h', 'v', isverbose);
				  }
				  return _OPT;
				}

<MS>H				{ BEGIN(HS);
				  if (!postscan)
					CHKCONFLICT('H', 'h', ishelp);
				  return _HSTOPT;
				}

<MS>i				{ if (!postscan) {
					isinstall = TRUE;
					CHKCONFLICT('i', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('i', 'd', isdisable);
					CHKCONFLICT('i', 'h', ishelp);
					CHKCONFLICT('i', 'k', iskillpiktd);
					CHKCONFLICT('i', 'K', iskillpiktc_svc);
					CHKCONFLICT('i', 'l', islist);
					CHKCONFLICT('i', 'R', isrestartpiktc_svc);
					CHKCONFLICT('i', 's', isshow);
					CHKCONFLICT('i', 't', isdelete);
				  }
				  return _OPT;
				}

<MS>k				{ if (!postscan) {
					iskillpiktd = TRUE;
					CHKCONFLICT('k', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('k', 'h', ishelp);
					CHKCONFLICT('k', 'i', isinstall);
					CHKCONFLICT('k', 'l', islist);
					CHKCONFLICT('k', 'r', isrestartpiktd);
					CHKCONFLICT('k', 's', isshow);
				  }
				  return _OPT;
				}

<MS>K				{ if (!postscan) {
					iskillpiktc_svc = TRUE;
					CHKCONFLICT('K', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('K', 'd', isdisable);
					CHKCONFLICT('K', 'e', isenable);
					CHKCONFLICT('K', 'h', ishelp);
					CHKCONFLICT('K', 'i', isinstall);
					CHKCONFLICT('K', 'l', islist);
					CHKCONFLICT('K', 'r', isrestartpiktd);
					CHKCONFLICT('K', 'R', isrestartpiktc_svc);
					CHKCONFLICT('K', 's', isshow);
					CHKCONFLICT('K', 't', isdelete);
				  }
				  return _OPT;
				}

<MS>l				{ if (!postscan) {
					islist = TRUE;
					CHKCONFLICT('l', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('l', 'd', isdisable);
					CHKCONFLICT('l', 'e', isenable);
					CHKCONFLICT('l', 'h', ishelp);
					CHKCONFLICT('l', 'i', isinstall);
					CHKCONFLICT('l', 'k', iskillpiktd);
					CHKCONFLICT('l', 'K', iskillpiktc_svc);
					CHKCONFLICT('l', 'R', isrestartpiktc_svc);
					CHKCONFLICT('l', 's', isshow);
				  }
				  return _OPT;
				}

<MS>A				{ BEGIN(AS);
				  if (!postscan) {
					CHKCONFLICT('A', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('A', 'h', ishelp);
				  }
				  return _ALTOPT;
				}

<MS>Af				{ BEGIN(AS);
				  if (!postscan) {
					CHKCONFLICT('A', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('A', 'h', ishelp);
					stripalm = FALSE;
				  }
				  return _ALTOPT;
				}

<MS>r				{ if (!postscan) {
					isrestartpiktd = TRUE;
					CHKCONFLICT('r', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('r', 'h', ishelp);
					CHKCONFLICT('r', 'k', iskillpiktd);
					CHKCONFLICT('r', 'K', iskillpiktc_svc);
					CHKCONFLICT('r', 'l', islist);
				  }
				  return _OPT;
				}

<MS>R				{ if (!postscan) {
					isrestartpiktc_svc = TRUE;
					CHKCONFLICT('R', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('R', 'd', isdisable);
					CHKCONFLICT('R', 'e', isenable);
					CHKCONFLICT('R', 'h', ishelp);
					CHKCONFLICT('R', 'i', isinstall);
					CHKCONFLICT('R', 'K', iskillpiktc_svc);
					CHKCONFLICT('R', 'l', islist);
					CHKCONFLICT('R', 's', isshow);
					CHKCONFLICT('R', 't', isdelete);
				  }
				  return _OPT;
				}

<MS>s				{ if (!postscan) {
					isshow = TRUE;
					CHKCONFLICT('s', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('s', 'd', isdisable);
					CHKCONFLICT('s', 'e', isenable);
					CHKCONFLICT('s', 'h', ishelp);
					CHKCONFLICT('s', 'i', isinstall);
					CHKCONFLICT('s', 'k', iskillpiktd);
					CHKCONFLICT('s', 'K', iskillpiktc_svc);
					CHKCONFLICT('s', 'l', islist);
					CHKCONFLICT('s', 'R', isrestartpiktc_svc);
					CHKCONFLICT('s', 't', isdelete);
				  }
				  return _OPT;
				}

<MS>t				{ if (!postscan) {
					isdelete = TRUE;
					CHKCONFLICT('t', 'c', (ischeck || isdoublecheck));
					CHKCONFLICT('t', 'e', isenable);
					CHKCONFLICT('t', 'h', ishelp);
					CHKCONFLICT('t', 'i', isinstall);
					CHKCONFLICT('t', 'K', iskillpiktc_svc);
					CHKCONFLICT('t', 'R', isrestartpiktc_svc);
					CHKCONFLICT('t', 's', isshow);
				  }
				  return _OPT;
				}

<MS>T				{ if (!postscan) {
					istest = TRUE;
					/* isdebug = TRUE; */
				  }
				  return _OPT;
				}

<MS>v				{ if (!postscan) {
					isverbose = TRUE;
					CHKCONFLICT('v', 'h', ishelp);
				  }
				  return _OPT;
				}

<MS>{W}				{ BEGIN(INITIAL); }

<MS>.				{ if (!postscan) {	/* should never get
							   to postscan
							 */
					sprintf(errmsg, "illegal option: %c", yytext[0]);
					cmderror(errmsg);
					bailout = TRUE;
				  }
				  return _OPT;
				}

<AS,PS,FS,OS,HS>all		{ return _ALL; }

<INITIAL>ALL			{ for (ii=strlen(ALL)-1; ii>=0; ii--)
					unput(ALL[ii]);
				}

<DS>{DL}[^ \t]*			{ cmdlval.sval = strdup(yytext);
				  return _DEFINE;
				}

<AS>{DL}[^ \t]*			{ cmdlval.sval = strdup(yytext);
				  return _ALERT;
				}

<PS>{DL}[^ \t]*			{ cmdlval.sval = strdup(yytext);
				  return _PROGRAM;
				}

<FS>{DL}[^ \t]*			{ cmdlval.sval = strdup(yytext);
				  return _FILE;
				}

<OS>{DL}[^ \t]*			{ cmdlval.sval = strdup(yytext);
				  return _OBJECT;
				}

<HS>{DL}[^ \t]*			{ cmdlval.sval = strdup(yytext);
				  return _HOST;
				}
<DS,AS,PS,FS,OS,HS>{W}		{ ; }
<DS,AS,PS,FS,OS,HS>.		{ return yytext[0]; }

.				{ return yytext[0]; }


%%

/* cmd error handler
 */
void
cmderror (const char *message)
{

	fprintf(stderr, "%s (in cmdparse()): %s\n", message, yytext);
	bailout = TRUE;
	return;
}

/* feed command-line arguments to the lexer input stream
 */
int
cmdinput (void)
{
	char c;

	if (argvcurr > argvlast)
		return(EOF);
	if ((c = (*argvcurr)[stroffset++]) != '\0')
		return(c);
	/* end of argument, move to the next */
	argvcurr++;
	stroffset = 0;
	return(' ');
}
