/*
   common.h -- PIKT common header file for piktc & piktc_svc

   $Header: /home/cvs/ucgsb/pikt/src/piktc/common.h,v 1.15 1999/05/09 18:39:48 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <sys/types.h>
#ifndef FILE
#include <stdio.h>
#endif

#define MAXLINE		512	/* max line length in various contexts */

#define MAXCMD		MAXLINE	/* max command length */

#define MAXBUF		65536	/* max length of function buffer string */
#define MAXPATH		256	/* max length of file path */
#define MAXIDENT	256	/* max length of ident */

#define MAXIFS		64	/* applies to both "if" & "#if",
				   also to "#include"
				 */
#define MAXWHILES	MAXIFS
#define MAXREPEATS	MAXIFS
#define MAXFORS		MAXIFS
#define MAXSWITCHS	MAXIFS
#define MAXLOOPS	3*MAXIFS

#define MODE_PRG_DEFAULT	"750"	/* default mode for installed
					   programs
					 */
#define MODE_FIL_DEFAULT	"640"	/* default mode for installed
					   files
					 */
#define UID_DEFAULT		"0"	/* default uid for installed
					   programs/files
					 */
#define GID_DEFAULT		"0"	/* default gid for installed
					   programs/files
					 */

#define ERROR		-1
#define NONE		-2
#define DENIED		-3	/* denied piktc service request */
#define UNAUTHORIZED	-4	/* unauthorized piktc service request */
#define IGNORE		-5

#define NIL		NONE	/* for value unknown */

#define ADD		1
#define DELETE		0

#ifndef FALSE
#define FALSE		0
#endif
#ifndef TRUE
#define TRUE		1
#endif

/* dat type */
#define STRING		0
#define NUMBER		1
#define FILEHANDLE	2

/* the order of these is vitally important! */
#define OPS		0
#define HST		1
#define GRP		2
#define ALS		3
#define DEF		4
#define MCR		5
#define ALM		6
#define ALT		7
#define PRG		8
#define FIL		9
#define OBJ		10

/* the order of these is vitally important! */
#define SYSCFG		0
#define DEFCFG		1
#define MCRCFG		2
#define ALMCFG		3
#define ALTCFG		4
#define PRGCFG		5
#define FILCFG		6
#define OBJCFG		7

/* statement type */
#define DO		1
#define IF		2
#define ELIF		3
#define ELSE		4
#define ENDIF		5
#define WHILE		6
#define ENDWHILE	7
#define WHILEAUX	107
#define REPEAT		8
#define UNTIL		9
#define REPEATAUX	109
#define FOR		10
#define ENDFOR		11
#define FORAUX		111
#define BREAK		12
#define CONTINUE	13
#define SWITCH		14
#define CASE		15
#define DEFAULT		16
#define BREAKSWITCH	17
#define ENDSWITCH	18

#define AGAIN		19
#define REDO		20
#define NEXT		21
#define SKIP		22
#define LAST		23
#define PAUSE		24
#define QUIT		25
#define DIE		26

#define OUTPUT		27
#define MAIL		28
#define PRINT		29
#define LOCLOG		30
#define PIKTLOG		31
#define SYSLOG		32

#define SET		33
#define EXEC		34
#define EXECWAIT	35

/* rul type */
#define BEG		1
#define RUL		2
#define END		3

/* alarm level -- correspond to syslog levels */
#define EMERG		0
#define ALERT		1
#define CRIT		2
#define ERR		3	/* also used for log type */
#define WARNING		4	/* also used for log type */
#define NOTICE		5
#define INFO		6	/* also used for log type */
#define DEBUG		7	/* also used for log type */

#define KILL_PIKTD_SERVICE		0x00000001
#define KILL_PIKTC_SVC_SERVICE		0x00000002
#define KILL_SERVICE			KILL_PIKTD_SERVICE | KILL_PIKTC_SVC_SERVICE

#define RESTART_PIKTD_SERVICE		0x00000004
#define RESTART_PIKTC_SVC_SERVICE	0x00000008
#define RESTART_SERVICE			RESTART_PIKTD_SERVICE | RESTART_PIKTC_SVC_SERVICE

#define DISABLE_ALERTS_SERVICE		0x00000010
#define ENABLE_ALERTS_SERVICE		0x00000020
#define SHOW_ALERTS_STATUS_SERVICE	0x00000040
/* #define_SERVICE????			0x00000080 */

#define INSTALL_ALERTS_SERVICE		0x00000100
#define INSTALL_PROGRAMS_SERVICE	0x00000200
#define INSTALL_FILES_SERVICE		0x00000400
#define INSTALL_OBJECTS_SERVICE		0x00000800
#define INSTALL_SERVICE			INSTALL_ALERTS_SERVICE | \
					INSTALL_PROGRAMS_SERVICE | \
					INSTALL_FILES_SERVICE | \
					INSTALL_OBJECTS_SERVICE

#define DELETE_ALERTS_SERVICE		0x00001000
#define DELETE_PROGRAMS_SERVICE		0x00002000
#define DELETE_FILES_SERVICE		0x00004000
#define DELETE_OBJECTS_SERVICE		0x00008000
#define DELETE_SERVICE			DELETE_ALERTS_SERVICE | \
					DELETE_PROGRAMS_SERVICE | \
					DELETE_FILES_SERVICE | \
					DELETE_OBJECTS_SERVICE

#define LIST_ALERTS_SERVICE		0x00010000
#define LIST_PROGRAMS_SERVICE		0x00020000
#define LIST_FILES_SERVICE		0x00040000
#define LIST_OBJECTS_SERVICE		0x00080000
#define LIST_SERVICE			LIST_ALERTS_SERVICE | \
					LIST_PROGRAMS_SERVICE | \
					LIST_FILES_SERVICE | \
					LIST_OBJECTS_SERVICE

#define ALL_SERVICES			0xffffffff

#define SHELL		"/bin/sh"

#define foreach(a,b)	for (a=b; a; a=a->next)
#define odd(a)		(a % 2) ? TRUE : FALSE
#define even(a)		(a % 2) ? FALSE : TRUE

#define FREE(a)		if (a) free(a)

/* used to record command-line alert/program/file/object/host hames;
   alert/program/file/object names in cfg.c; file names passing from piktc
   to piktc_svc
 */
typedef struct _nam {
	char *name;
	char *aux;		/* only used by alerts/programs/files */
	char *mode;		/* only used by programs/files for mode */
	char *uid;		/* only used by programs/files for uid */
	char *gid;		/* only used by programs/files for gid */
	int state;		/* ADD/DELETE/ERROR */
	struct _nam *next;
} nam;

/* used when passing files from piktc to piktc_svc
 */
typedef struct _fil {
	char *name;		/* filename */
	char *directory;	/* file directory */
	char *extension;	/* file extension, e.g., .alt */
	char *mode;		/* file permissions */
	char *uid;		/* file uid */
	char *gid;		/* file gid */
	int size;		/* file size in bytes */
	char *text;		/* file contents as one long char string */
} fil;

/* used to store lines from all .cfg files */
typedef struct _cfgln {
	int lineno;		/* line number */
	char *file;		/* line's cfg file */
	unsigned long *bs;	/* line's host bitset "tag" */
	char *line;		/* line text */
	int len;		/* line length */
	int domcr;		/* macro expand this line? */
	int doproc;		/* process this line? */
	struct _cfgln *next;
} cfgln;

typedef struct _def {
	char *component;	/* macro/alarm def component */
	struct _def *next;
} def;

/* note that not all fields are used for every ident */
typedef struct _ident {
	char *name;		/* ident name */
	int type;		/* DEF/OPS/HST/GRP/ALS/MCR/ALM/ALT/PRG/FIL/OBJ */
	char *alias;		/* only used by ALS type */
	unsigned long *bs;	/* ident's host/define bitset "tag" */
	def *definition;	/* list of ident defs */
	struct _ident *next;
} ident;

/* dir/file paths */
extern char *home,		/* PIKT home directory */
            *bindir,		/* binaries directory */
            *etcdir,		/* admin directory */
            *libdir,		/* lib directory */
            *vardir,		/* var directory */
            *altdir,		/* alerts directory */
            *prgdir,		/* programs directory */
            *objdir,		/* objects directory */
            *cfgdir,		/* configs directory */
            *stgdir,		/* staging directory */
            *hstdir,		/* histories directory */
            *logdir,		/* log directory */
            *PIKTconf,		/* PIKT conf file */
            *pikt,		/* pikt program */
            *piktd,		/* piktd program */
            *piktc,		/* piktc program */
            *piktc_svc,		/* piktc_svc program */
            *piktdconf,		/* piktd conf file */
            *piktdconftmp,	/* piktd conf temp file */
            *piktdlock,		/* piktd lock file */
            *piktdlog,		/* piktd log file */
            *piktdalt,		/* piktd alerts file */
            *piktclock,		/* piktc lock file */
            *piktclog,		/* piktc log file */
            *piktc_svclock,	/* piktc_svc lock file */
            *piktc_svclog;	/* piktc_svc log file */

/* command-line options */
extern int ischeck,
           isdebug,
           isdelete,
           isdoublecheck,
           isdisable,
           isenable,
           ishelp,
           isinstall,
           iskillpiktd,
           iskillpiktc_svc,
           islist,
           isrestartpiktd,
           isrestartpiktc_svc,
           isshow,
           istest,
           isverbose;

extern int bailout;	/* signifies command-line error,
			   or error in systems.cfg
			 */

extern const char piktc_version[];
extern const char piktc_svc_version[];

extern int uid;			/* piktc/piktc_svc service request uid */
extern char *master;		/* hostname of master control system */
extern char *domain;		/* local network domain, or "domain" */
extern char *accessmode;	/* "local" or "global" */

extern char hostname[];		/* hostname of system that piktc and/or
				   piktc_svc runs on
			 	 */
extern char domainname[];	/* domainname of system that piktc and/or
				   piktc_svc runs on
			 	 */

extern int prescan;	/* is this cfg file scan preliminary or not? */
extern int postscan;	/* is this cmd line scan secondary or not? */

extern int cfgerr;	/* signifies syntax error in any config file */
extern int stzerr;	/* signifies syntax error in current stanza */
extern int lineno;	/* line number in config file */

extern int errno;

extern char **argvcurr;	/* current command-line argument */
extern char **argvlast;	/* last command-line argument */

extern nam *alerts, *cfgalerts, *incmdalerts, *excmdalerts, *cmdalerts,
           *programs, *cfgprograms, *incmdprograms, *excmdprograms, *cmdprograms,
           *files, *cfgfiles, *incmdfiles, *excmdfiles, *cmdfiles,
           *objects, *cfgobjects, *incmdobjects, *excmdobjects, *cmdobjects,
                     *cfghosts,   *incmdhosts,   *excmdhosts,   *cmdhosts;

extern int alertsall, programsall, filesall, objectsall, hostsall;

extern FILE *cfgout;

extern const char *cfgfilenames[];	/* systems.cfg/defines.cfg/macros.cfg/
					   alarms.cfg/alerts.cfg/programs.cfg/
					   files.cfg/objects.cfg
					 */
extern char *cfgfilecurr;	/* name of current config file */
extern char *cfgfileprev;	/* name of previous config file */

extern char *infile;		/* current input infile */

extern const char *cfgbof[];	/* signifies beginning of new config file */
extern char *cfp;

extern int cfgcurr;		/* current config file */
extern int cfglast;		/* last config file to be processed */
extern cfgln *configs[];	/* array of pointers to cfg file lines */
extern cfgln *clplast;		/* pointer to last cfg line */

extern ident *gidp;			/* points to active ident */
extern ident *idents[];			/* ident hash table */
extern const char *identypes[];		/* OPS/HST/GRP/ALS/DEF/MCR/ALM/ALT/PRG/FIL/OBJ */

extern size_t hbslongs;			/* number of longs in a host bit-set */
extern size_t dbslongs;			/* number of longs in a define bit-set */

extern unsigned long *ghbs;		/* global host bitset */

extern unsigned long *hbsfull, *hbsempty; /* full and empty host bitsets;
					     currently serve no purpose
					   */

extern unsigned long *hbscurr;		/* host bitset for the current
					     piktc command
					 */

extern unsigned long *gdbs;		/* global define bitset */
extern unsigned long *gubs;		/* global undefined bitset */

extern unsigned long *dbsfull, *dbsempty; /* full and empty define bitsets;
					     currently serve no purpose
					   */

extern unsigned long *dbscurr;		/* define bitset for the current
					   piktc command
					 */

extern char **hosts;			/* system hostnames */
extern int hostnum;			/* the number of configured hosts */
extern char *hostcurr;			/* the current hostname */

extern char **opsys;			/* operating systems */
extern int opsysnum;			/* the number of configured os'es */

extern int defnum;			/* the number of configured defines */

extern unsigned stroffset;		/* offset from beginning of string */
extern unsigned bof;			/* beginning of file */
extern unsigned bon;			/* beginning of (line) number */
extern unsigned bol;			/* beginning of line */

extern def *defcurr;			/* current definition */

extern char *mins, *hrs, *dom, *mons, *dow;	/* timing parameters */
extern char *priority;				/* alert nice level */
extern char *nicecmd, *mailcmd, *lpcmd;		/* strings written to
						   piktd.conf
						 */

extern int stripalm, stripprg, stripfil, stripobj;	/* strip these files
							   of comments?
							 */

extern int domacros;			/* do macro processing? */
extern int dostrip;			/* do comment stripping? */

extern void *incfiles;			/* include files */
extern int incnum;			/* the current (nested) include
					   file number
					 */

extern unsigned long services;		/* piktc-to-piktc_svc access privileges */

/* init functions */
extern int chkpiktc_svc(char *host);
extern int piktc_svc_main(void);
extern void init(int, char **, void (*)(void));
extern void getconf(void);

/* command-line functions */
extern void piktc_usage(void);
extern void piktc_svc_usage(void);
extern void preprocesscmd(void);
extern nam *regname(nam **, char *, int);
extern void retire(nam **);
extern nam *lookupname(nam *, char *);

/* lexer and parser functions */
extern int cmdparse(void);
extern int sysparse(void);
extern int tagparse(void);
extern int defparse(void);
extern int cfgparse(void);
extern int cmdlex(void);
extern int syslex(void);
extern int taglex(void);
extern int deflex(void);
extern int cfglex(void);
extern void cmderror(const char *);
extern void syserror(const char *);
extern void tagerror(const char *);
extern void deferror(const char *);
extern void cfgerror(const char *);
extern int striplex(void);

/* ident functions */
extern ident *lookupident(char *, int);
extern void regident(char *, int, int);
void cleanup(void);
extern char *alias(char *);
extern void makepiktmacros(void);
extern char *getnicecmd(char *);

/* bitset functions */
extern void empty(unsigned long *, size_t);
extern void fill(unsigned long *, size_t);
extern unsigned long *bsunion(unsigned long *, unsigned long *, size_t);
extern unsigned long *bsintersection(unsigned long *, unsigned long *, size_t);
extern unsigned long *bscomplement(unsigned long *, size_t);
extern int bsmember(unsigned long *, unsigned long *, size_t);
extern int bssubset(unsigned long *, unsigned long *, size_t);
extern void bscopy(unsigned long *, unsigned long *, size_t);
extern unsigned long *bsdup(unsigned long *, size_t);
extern unsigned long *getbs(char *, int);
extern void tag(int, char *, unsigned long *, size_t, int, int, char *);
extern ident *insert(char *, int);

/* alt/prg/fil/obj file functions */
extern void regcomp(char *);
extern void beginalarms(char *);
extern void writealarm(char *);
extern void endalarms(char *);
extern void beginfile(char *, char *, char *, char *, nam **);
extern void writefile(char *);
extern void endfile(char *);
extern void beginobjects(char *);
extern void writeobject(char *);
extern void endobjects(char *);
extern int conflinelen(char *);

/* rpc functions */
extern void enablealerts(char *, nam *, int);
extern void disablealerts(char *, nam *, int);
extern void installfiles(char *, nam *, char *, const char *, char *, char *, char *, int);
extern void deletefiles(char *, nam *, char *, const char *, int);
extern void listfiles(char *, nam *, char *, const char *, int);
extern void killdaemon(char *, unsigned long, int);
extern void restartdaemon(char *, unsigned long, int);
extern void showstata(char *, nam *, int);

/* lock functions */
extern int islock(char *);
extern int mklock(char *, char *, int, int);
extern int rmlock(char *, int, int);
extern int rmlocks(int, int);

/* logging functions */
extern void logreturn(int, int, const char *, int, int, const char *, ...);
extern void logexit(int, int, const char *, int, int, const char *, ...);

/* signal functions */
extern void sig_init(void);

/* debugging functions */
extern void dumpcfgs(unsigned long *);
