%{

/*
   def.y -- PIKT piktc def parser:  deals with define conditionals and the
            marking of config file lines for processing; marking does not
            apply to systems.cfg!

   $Header: /home/cvs/ucgsb/pikt/src/piktc/def.y,v 1.4 1999/05/02 08:01:38 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "common.h"

#define DBSEMPTY	(unsigned long *) calloc(dbslongs, sizeof(unsigned long))

#define CHKIFNUM	if (ifnum > MAXIFS) \
				logexit(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in defparse(), %s, line %d, MAXIFS (%d) exceeded", cfgfilecurr, lineno-1, MAXIFS); \
			else if (ifnum < 0) \
				logexit(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in defparse(), %s, line %d, ifnum < 0", cfgfilecurr, lineno-1)

#define SETBS(a,b)	tbs1 = a; \
			a = b; \
			FREE(tbs1)

#define FREE_BS		FREE(dbscur[ifnum]); \
			FREE(dbscum[ifnum]); \
			FREE(ubscur[ifnum]); \
			FREE(ubscum[ifnum])

#define DOPROC		bssubset(gdbs, dbscur[ifnum], dbslongs) && bssubset(gubs, ubscur[ifnum], dbslongs)

#define DUMPBS		logreturn(PIKTLOG, DEBUG, __FILE__, __LINE__, FALSE, "gdbs %x, dbscur[ifnum-1] %x, dbscur[ifnum] %x, gubs %x, ubscur[ifnum-1] %x, ubscur[ifnum] %x", *gdbs, *dbscur[ifnum-1], *dbscur[ifnum], *gubs, *ubscur[ifnum-1], *ubscur[ifnum])

int ifnum;

int doproc[MAXIFS+1];

unsigned long *dbscur[MAXIFS+1], *dbscum[MAXIFS+1];
unsigned long *ubscur[MAXIFS+1], *ubscum[MAXIFS+1];
unsigned long *tbs1, *tbs2;

%}

%union {
	char *sval;
	int ival;
	cfgln *clpval;
	unsigned long *dbsval;
}

%token _DEFCFGBOF _MCRCFGBOF _ALMCFGBOF _ALTCFGBOF _PRGCFGBOF _FILCFGBOF _OBJCFGBOF
%token _DEF _UNDEF _IFDEF _IFNDEF _ELIFDEF _ELIFNDEF _ELSEDEF _ENDIFDEF
%token <clpval> _LINE
%token <sval> _STANZA
%token <sval> _IDENT
%token _TRUE _FALSE
%type <dbsval> ident

%%

start:		_DEFCFGBOF cfgdefs
		| _MCRCFGBOF {
			ifnum = 0;
			doproc[ifnum] = TRUE;
			dbscur[ifnum] = DBSEMPTY;
			dbscum[ifnum] = DBSEMPTY;
			ubscur[ifnum] = DBSEMPTY;
			ubscum[ifnum] = DBSEMPTY;
		} lines
		_ALMCFGBOF lines
		_ALTCFGBOF lines
		_PRGCFGBOF lines
		_FILCFGBOF lines
		_OBJCFGBOF lines
		{
			FREE_BS;
		}
		;

cfgdefs:	cfgdef
		| cfgdefs cfgdef
		;

cfgdef:		_STANZA _TRUE {
			regident($1, DEF, TRUE);
		}
		| _STANZA _FALSE {
			regident($1, DEF, FALSE);
		}
		;

lines:		line
		| lines line
		;

line:		_LINE {
			if ($1 && ($1->line[0] != '#'))	$1->doproc = doproc[ifnum];
		}
		| define
		| conditional
		;

define:		_DEF ident {
			SETBS(gdbs, bsunion(gdbs, $2, dbslongs));
			tbs2 = bscomplement($2, dbslongs);
			SETBS(gubs, bsintersection(gubs, tbs2, dbslongs));
			free(tbs2);
		}
		| _UNDEF ident {
			SETBS(gubs, bsunion(gubs, $2, dbslongs));
			tbs2 = bscomplement($2, dbslongs);
			SETBS(gdbs, bsintersection(gdbs, tbs2, dbslongs));
			free(tbs2);
		}
		;

conditional:	if_def _ENDIFDEF {
			FREE_BS;
			ifnum--;
			CHKIFNUM;
		}
		| if_def else_def _ENDIFDEF {
			FREE_BS;
			ifnum--;
			CHKIFNUM;
		}
		| if_def elif_defs _ENDIFDEF {
			FREE_BS;
			ifnum--;
			CHKIFNUM;
		}
		| if_def elif_defs else_def _ENDIFDEF {
			FREE_BS;
			ifnum--;
			CHKIFNUM;
		}
		;

if_def:		ifdef
		| ifdef	lines
		| ifndef
		| ifndef lines
		;

elif_defs:	elifdef
		| elifdef lines
		| elifndef
		| elifndef lines
		| elif_defs elifdef
		| elif_defs elifdef lines
		| elif_defs elifndef
		| elif_defs elifndef lines
		;

else_def:	elsedef
		| elsedef lines
		;

ifdef:		_IFDEF ident {
			ifnum++;
			CHKIFNUM;
			dbscur[ifnum] = bsunion($2, dbscur[ifnum-1], dbslongs);
			ubscur[ifnum] = bsdup(ubscur[ifnum-1], dbslongs);
			dbscum[ifnum] = bsdup($2, dbslongs);
			ubscum[ifnum] = DBSEMPTY;
			doproc[ifnum] = DOPROC;
		}
		;

ifndef:		_IFNDEF ident {
			ifnum++;
			CHKIFNUM;
			dbscur[ifnum] = bsdup(dbscur[ifnum-1], dbslongs);
			ubscur[ifnum] = bsunion($2, ubscur[ifnum-1], dbslongs);
			dbscum[ifnum] = DBSEMPTY;
			ubscum[ifnum] = bsdup($2, dbslongs);
			doproc[ifnum] = DOPROC;
		}
		;

elifdef:	_ELIFDEF ident {
			SETBS(ubscur[ifnum], bsunion(dbscur[ifnum], ubscur[ifnum-1], dbslongs));
			SETBS(dbscur[ifnum], bsunion($2, dbscur[ifnum-1], dbslongs));
			SETBS(dbscum[ifnum], bsunion($2, dbscum[ifnum], dbslongs));
			doproc[ifnum] = DOPROC;
		}
		;

elifndef:	_ELIFNDEF ident {
			SETBS(dbscur[ifnum], bsunion(ubscur[ifnum], dbscur[ifnum-1], dbslongs));
			SETBS(ubscur[ifnum], bsunion($2, ubscur[ifnum-1], dbslongs));
			SETBS(ubscum[ifnum], bsunion($2, ubscum[ifnum], dbslongs));
			doproc[ifnum] = DOPROC;
		}
		;

elsedef:	_ELSEDEF {
			SETBS(dbscur[ifnum], bsunion(ubscum[ifnum], dbscur[ifnum-1], dbslongs));
			SETBS(ubscur[ifnum], bsunion(dbscum[ifnum], ubscur[ifnum-1], dbslongs));
			doproc[ifnum] = DOPROC;
		}
		;

ident:		_IDENT {
			$$ = getbs($1, DEF);
			FREE($1);
		}
		;

%%
