%{

/* 
   tag.l -- PIKT piktc tag lexer:  deals with machine conditionals and the
            tagging of config file lines; tagging does not apply to
            systems.cfg!

   $Header: /home/cvs/ucgsb/pikt/src/piktc/tag.l,v 1.12 1999/05/02 08:01:39 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "common.h"
#include "tag.tab.h"

char *cp;

typedef struct _filstk {
	char *file;
	YY_BUFFER_STATE bs;
	int lineno;
	struct _filstk *next;
} filstk;

void *incfiles;
int incnum;

char *incfile = (char *) NULL;

int fd[MAXIFS+1][2];
FILE *stripin, *stripout;

void begintagfile(char *);
void endtagfile(void);
void pushtagfile(char *, YY_BUFFER_STATE, int);
void poptagfile(char **, YY_BUFFER_STATE *, int *);

%}

W		[ \t]
D		[0-9]
L		[A-Za-z]
DL		[0-9A-Za-z]

%x REM COND INC

%%

<*>^#(fix|freeze){W}*(\177)?\n	{ lineno++;
				  if (!prescan)
					domacros = FALSE;
				}
<*>^#(unfix|unfreeze){W}*(\177)?\n? { lineno++;
				  if (!prescan)
					domacros = TRUE;
				}

<*>^#include{W}+\<[^\>]+\>{W}*(\177)?\n? { lineno++;
				  *strrchr(yytext, '>') = '\0';
				  cp = strchr(yytext, '<') + 1;
				  /* linux chokes on this
				  FREE(incfile);
				   */
				  incfile = (char *) malloc((strlen(cfgdir)+strlen(cp)+2) * sizeof(char));
				  sprintf(incfile, "%s/%s", cfgdir, cp);
				  begintagfile(incfile);
				}

<*>^#include{W}+\"[^\"]+\"{W}*(\177)?\n? { lineno++;
				  *strrchr(yytext, '\"') = '\0';
				  cp = strchr(yytext, '\"') + 1;
				  /* linux chokes on this
				  FREE(incfile);
				   */
				  incfile = strdup(cp);
				  begintagfile(incfile);
				}

<*>^#if{W}+			{ BEGIN(COND);
				  return _IF;
				}
<*>^#(elif|elsif|elseif){W}+	{ BEGIN(COND);
				  return _ELIF;
				}
<*>^#else{W}+			{ BEGIN(COND);
				  return _ELSE;
				}
<*>^#else(\177)?\n		{ lineno++;
				  return _ELSE;
				}
<*>^#(endif|fi){W}+		{ BEGIN(COND);
				  return _ENDIF;
				}
<*>^#(endif|fi)(\177)?\n?	{ lineno++;
				  return _ENDIF;
				}
<*>^#.?				{ BEGIN(REM);
				  yyless(0);
				}

<*>^ERR.*\n?			{ BEGIN(REM);
				  yyless(3);
				}

<REM>.*\n?			{ if ((cp = strchr(yytext,'\n')))
					*cp = '\0';
				  tagerror("syntax error");
				  lineno++;
				}

<COND,INC>[^!&|() \t\n\177]+	{ taglval.sval = strdup(yytext);
				  return _IDENT;
				}
<COND,INC>"!"|"&"|"|"|"("|")"	{ return yytext[0]; }
<COND,INC>{W}+			{ ; }
<COND,INC>(\177)?\n		{ BEGIN(INITIAL);
				  lineno++;
				}

<*>^{W}*\n?			{ lineno++;
				  if ((yyleng >1) && ((cfgcurr == PRGCFG) || (cfgcurr == FILCFG))) {
					taglval.sval = strdup("\t\n");
					return _LINE;
				  }
				}

<*>\177\n?			{ lineno++; }

<*>^#(def|define|undef|undefine|ifdef|ifndef|elifdef|elsifdef|elseifdef|elifndef|elsifndef|elseifndef|elsedef|endifdef|fidef)[^\n\177]*(\177)?\n? {
				  lineno++;
				  /* dispose of any '\177' */
				  if ((cp = strchr(yytext, '\177'))) {
					*cp = *(cp+1);
					*(cp+1) = '\0';
				  }
				  taglval.sval = strdup(yytext);
				  return _DEF;
				}

<*>^[^#\n\177][^\n\177]*(\177)?\n? {
				  lineno++;
				  /* dispose of any '\177' */
				  if ((cp = strchr(yytext, '\177'))) {
					*cp = *(cp+1);
					*(cp+1) = '\0';
				  }
				  taglval.sval = strdup(yytext);
				  return _LINE;
				}

%%

/* tag error handler
 */
void
tagerror (const char *message)
{

	fprintf(stderr, "%s, at or near line %d: %s (in tagparse()): %s\n", cfgfilecurr, lineno, message, yytext);
	bailout = TRUE;
	if (!incfiles)
		BEGIN(INITIAL);
	return;
}

/* tag end-of-file handler
 */
int
tagwrap (void)
{

	if (incfiles) {
		endtagfile();
		return 0;
	} else
		return 1;
}

/* push the current input buffer onto the file stack; create and switch to
   a new input buffer; open the new include file and start feeding its
   contents to the new input buffer; if stripping comments from the new
   include file, spawn a child process to pass the include file
   contents to the strip lexer
 */
void
begintagfile (char *file)
{
	pid_t pid;
	struct stat fbuf;

	if (stat(file, &fbuf) != 0) {
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in begintagfile(), stat() error for %s", file);
	}
	pushtagfile(infile, YY_CURRENT_BUFFER, lineno);
	cfgfileprev = cfgfilecurr;
	cfgfilecurr = infile = strdup(file);
	yy_switch_to_buffer(yy_create_buffer(tagin, MAXBUF));
	if (!prescan)
		domacros = TRUE;
	incnum++;
	if (incnum > MAXIFS)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in begintagfile(), maximum number (%d) of include files exceeded", MAXIFS);
	lineno = 1;
	BEGIN(INC);
	if (dostrip) {
		if (pipe(fd[incnum]) < 0)
			logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in begintagfile(), pipe() error");
		switch (pid = fork()) {
			case -1:	/* error */
				logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in begintagfile(), fork() error");
				break;
			/* pipe from child to parent */
			case 0:		/* child */
				close(fd[incnum][0]);
				stripin = fopen(infile, "r");
				stripout = fdopen(fd[incnum][1], "w");
				striplex();
				fclose(stripin);
				fclose(stripout);
				close(fd[incnum][1]);
				exit(0);
				break;
			default:	/* parent */
				close(fd[incnum][1]);
				#if defined(TIME_TO_SLEEP)
				sleep(TIME_TO_SLEEP);
				#endif
				tagin = fdopen(fd[incnum][0], "r");
				break;
		}
	} else {
		tagin = fopen(infile, "r");
	}
	return;
}

/* at the end of the current include file, close the file, delete its
   input buffer, pop the previous include file off the file stack, and
   switch to the previous input buffer
 */
void
endtagfile (void)
{
	YY_BUFFER_STATE bs;

	fclose(tagin);
	if (dostrip)
		close(fd[incnum][0]);
	incnum--;
	if (incnum < 0)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in endtagfile(), incnum < 0");
	yy_delete_buffer(YY_CURRENT_BUFFER);
	poptagfile(&infile, &bs, &lineno);
	cfgfileprev = cfgfilecurr;
	cfgfilecurr = infile;
	yy_switch_to_buffer(bs);
	if (!incfiles)
		BEGIN(INITIAL);
	return;
}

/* push current input buffer info onto the file stack
 */
void
pushtagfile (char *file, YY_BUFFER_STATE bs, int lno)
{
	filstk *fsp;

	fsp = (filstk *) malloc(sizeof(filstk));
	fsp->file = strdup(file);
	fsp->bs = bs;
	fsp->lineno = lno;
	fsp->next = (filstk *) incfiles;
	incfiles = fsp;
	return;
}

/* pop input buffer info off the file stack
 */
void
poptagfile (char **file, YY_BUFFER_STATE *bsp, int *lnp)
{
	static filstk *fsp;

	fsp = (filstk *) incfiles;
	/* linux chokes on this
	FREE(*file);
	 */
	*file = strdup(((filstk *) incfiles)->file);
	*bsp = ((filstk *) incfiles)->bs;
	*lnp = ((filstk *) incfiles)->lineno;
	incfiles = ((filstk *) incfiles)->next;
	/* redhat linux (but not slackware!) chokes on this
	FREE(fsp->file);
	 */
	FREE(fsp);
	return;
}

