%{

/*
   hst.l -- PIKT history file lexer

   $Header: /home/cvs/ucgsb/pikt/src/piktd/hst.l,v 1.5 1999/04/15 04:16:19 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "common.h"
#include "hst.tab.h"

int lineno;
int ii;

%}

W		[ \t]
D		[0-9]
L		[A-Za-z]
DL		[0-9A-Za-z]

%x LS VGS SS

%option noyywrap

%%

^A		{ return _ALM; }
^R		{ return _VAR; }
^K		{ return _KEY; }

{W}{D}{1,5}	{ hstlval.ival = atoi(yytext+1);
		  return _INT;
		}

^L		{ BEGIN(LS);
		  return _LOG;
		}
<LS>{W}{D}+	{ hstlval.lval = atol(yytext+1);
		  return _LNG;
		}
<LS,INITIAL>{W}[^0-9].*$ { hstlval.sval = strdup(yytext+1);
		  BEGIN(INITIAL);
		  return _STR;
		}

^V		{ ii = 0;
		  BEGIN(VGS);
		  return _VAL;
		}
^G		{ ii = 0;
		  BEGIN(VGS);
		  return _GLB;
		}
<VGS>{W}{D}{1,5} { hstlval.ival = atoi(yytext+1);
		  ii++;
		  if (ii >= 2)
			BEGIN(SS);
		  return _INT;
		}
<SS>{W}.*$	{ hstlval.sval = strdup(yytext+1);
		  BEGIN(INITIAL);
		  return _STR;
		}

\n		{ lineno++; }
.		{ return yytext[0]; }

%%

/* hst error handler
 */
void
hsterror (const char *message)
{
	static int l = 0;

	if (l != lineno)
		l = lineno;
	/* if lineno > 1, have successfully parsed the A line, hence have
	   set alarmcurr to match the current alarm history, and have set
	   alarmcurr->hsterr = 0; if lineno = 1, alarmcurr is undefined
	 */
	if (lineno > 1)
		HSTERR = 1;
	logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in .hst file, at or near line %d: %s (in hstparse()): @%s@", lineno, message, yytext);
	return;
}
